@@ -270,6 +270,10 @@ STYLE_RULES = {
         "Ensures there are no unnecessary leading or trailing commas in a"
         " matrix or cell."),
 
+    "spurious_row_semicolon" : Style_Rule(
+        "Ensures there are no unnecessary leading or trailing semicolons in a"
+        " matrix or cell."),
+
     "whitespace_colon" : Style_Rule(
         "Ensures there is no whitespace around colons except if they come"
         " after a comma."),
@@ -338,7 +342,13 @@ STYLE_RULES = {
             "tab_width": Integer_Style_Configuration(
                 "Number of spaces to indent by.",
                 lower_limit = 2,
-                default = 4)
+                default = 4),
+            "align_round_brackets" : Boolean_Style_Configuration(
+                "Align continuations inside normal brackets.",
+                default = True),
+            "align_other_brackets" : Boolean_Style_Configuration(
+                "Align continuations inside matrix and cell expressions.",
+                default = True)
         }),
 
     "redundant_brackets" : Style_Rule(
@@ -359,6 +369,12 @@ STYLE_CONFIGURATION = {
 }
 # A flat list of all style configuration items
 
+STYLE_CONFIGURATION["regex_tickets"] = Regex_Style_Configuration(
+    "Regex for tickets in your issue tracking system",
+    default = "")
+# Add the special regex that identifies tickets (not part of any
+# rule).
+
 METRICS = {
     "file_length" : File_Metric(
         "File lines",
