@@ -35,10 +35,11 @@ from abc import ABCMeta, abstractmethod
 from miss_hit_core import work_package
 from miss_hit_core import command_line
 from miss_hit_core import config
-from miss_hit_core import g_cfg
 
 from miss_hit_core.errors import (Location, Error, ICE,
-                                  Message_Handler, HTML_Message_Handler)
+                                  Message_Handler,
+                                  HTML_Message_Handler,
+                                  JSON_Message_Handler)
 from miss_hit_core.m_ast import *
 from miss_hit_core.m_lexer import MATLAB_Lexer, Token_Buffer
 from miss_hit_core.m_parser import MATLAB_Parser
@@ -365,6 +366,17 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
     statement_start_token = None
     current_indent = 0
     enclosing_ast = None
+    bracket_stack = []
+    relevant_brackets = set()
+
+    # Find out which brackets are relevant for us for aligning
+    # continuations
+    if cfg.active("indentation"):
+        if cfg.style_config["align_round_brackets"]:
+            relevant_brackets.add("BRA")
+        if cfg.style_config["align_other_brackets"]:
+            relevant_brackets.add("M_BRA")
+            relevant_brackets.add("C_BRA")
 
     for n, token in enumerate(tbuf.tokens):
         if n - 1 >= 0:
@@ -527,6 +539,13 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
                                    "whitespace",
                                    fixed)
 
+            if cfg.active("spurious_row_semicolon") and token.fix.spurious:
+                token.fix.delete = True
+                mh.style_issue(token.location,
+                               "this semicolon is not required and can "
+                               "be removed",
+                               fixed)
+
         elif token.kind == "COLON":
             if cfg.active("whitespace_colon"):
                 if prev_in_line and prev_in_line.kind == "COMMA":
@@ -793,6 +812,23 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
                         current_indent = enclosing_ast.get_indentation() + 1
                     offset = 0
 
+                elif bracket_stack and \
+                     bracket_stack[-1].kind in relevant_brackets and \
+                     token.kind != "ANNOTATION":
+                    # For stuff inside a bracket group we care about,
+                    # we align it with the opening brace + 1, except
+                    # for the closing brace, that one we align exactly
+                    # with the opening brace.
+                    if bracket_stack[-1].fix.correct_indent is not None:
+                        offset = bracket_stack[-1].fix.correct_indent - \
+                            statement_start_token.location.col_start
+                    else:
+                        offset = bracket_stack[-1].location.col_start - \
+                            statement_start_token.location.col_start
+
+                    if token.kind not in ("KET", "M_KET", "C_KET"):
+                        offset += 1
+
                 else:
                     # This is a continued line. We try to preserve
                     # the offset. We work out how much extra space
@@ -829,6 +865,16 @@ def stage_3_analysis(mh, cfg, tbuf, is_embedded, fixed):
                                     token.location.col_start),
                                    fixed)
 
+        # Keep track of matrix and cell expressions. Again, required
+        # for indentation. We do this *after* the fixing so we have a
+        # chance to deal with the closing brackets while knowing the
+        # opening brace, and the opening braces considering the
+        # context we're currently in.
+        if token.kind in ("M_BRA", "C_BRA", "BRA"):
+            bracket_stack.append(token)
+        elif token.kind in ("M_KET", "C_KET", "KET"):
+            bracket_stack.pop()
+
 
 class MH_Style_Result(work_package.Result):
     def __init__(self, wp):
@@ -935,11 +981,6 @@ class MH_Style(command_line.MISS_HIT_Back_End):
         except Error:
             parse_tree = None
 
-        # Create CFG for debugging purposes
-
-        if parse_tree and wp.options.debug_cfg:
-            g_cfg.debug_cfg(parse_tree, wp.mh)
-
         # Stage 3 - rules around individual tokens
 
         stage_3_analysis(
@@ -994,6 +1035,10 @@ def main_handler():
         "--html",
         default=None,
         help="Write report to given file as HTML")
+    clp["output_options"].add_argument(
+        "--json",
+        default=None,
+        help="Produce JSON report")
     clp["output_options"].add_argument(
         "--no-style",
         action="store_true",
@@ -1011,11 +1056,6 @@ def main_handler():
         action="store_true",
         default=False,
         help="Debug option to check AST links")
-    clp["debug_options"].add_argument(
-        "--debug-cfg",
-        action="store_true",
-        default=False,
-        help="Build CFG for every function")
 
     style_option = clp["ap"].add_argument_group("rule options")
 
@@ -1040,10 +1080,17 @@ def main_handler():
     options = command_line.parse_args(clp)
 
     if options.html:
+        if options.json:
+            clp["ap"].error("Cannot produce JSON and HTML at the same time")
         if os.path.exists(options.html) and not os.path.isfile(options.html):
             clp["ap"].error("Cannot write to %s: it is not a file" %
                             options.html)
         mh = HTML_Message_Handler("style", options.html)
+    elif options.json:
+        if os.path.exists(options.json) and not os.path.isfile(options.json):
+            clp["ap"].error("Cannot write to %s: it is not a file" %
+                            options.json)
+        mh = JSON_Message_Handler("style", options.json)
     else:
         mh = Message_Handler("style")
 
